/**********************************************************************
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
 civmap_ex.h - Copyright (C) 2002 - R. Wetmore

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#ifndef	FC__CIVMAP_EX_H
#define	FC__CIVMAP_EX_H

/**
 * civmap_ex.c - extended civmap functions (tile accessors, etc.)
 */

#include "civterrain.h"
#include "civtile.h"
#include "civtile_ex.h"
#include "civmap.h"

struct city;
struct unit;

CIVTILE *civmap_get_tile(const CIVMAP *pmap, int x, int y);

struct city *civmap_get_city(const CIVMAP *pmap, int x, int y);
void civmap_set_city(const CIVMAP *pmap, int x, int y, struct city *pcity);
struct city *civmap_get_worked(const CIVMAP *pmap, int x, int y);
void civmap_set_worked(const CIVMAP *pmap, int x, int y, struct city *pcity);
short civmap_get_continent(const CIVMAP *pmap, int x, int y);
void civmap_set_continent(const CIVMAP *pmap, int x, int y, int val);
short civmap_get_ocean(const CIVMAP *pmap, int x, int y);
void civmap_set_ocean(const CIVMAP *pmap, int x, int y, int val);

int civmap_goodness(const CIVMAP *pmap, int x, int y);
int civmap_get_food(const CIVMAP *pmap, int x, int y);
int civmap_get_food_base(const CIVMAP *pmap, int x, int y);
int civmap_get_shield(const CIVMAP *pmap, int x, int y);
int civmap_get_shield_base(const CIVMAP *pmap, int x, int y);
int civmap_get_trade(const CIVMAP *pmap, int x, int y);
int civmap_get_trade_base(const CIVMAP *pmap, int x, int y);
const char *civmap_get_fpt_text(const CIVMAP *pmap, int x, int y);
const char *civmap_get_info_text(const CIVMAP *pmap, int x, int y);

int civmap_road_time(const CIVMAP *pmap, int x, int y);
int civmap_railroad_time(const CIVMAP *pmap, int x, int y);
int civmap_mine_time(const CIVMAP *pmap, int x, int y);
int civmap_irrigation_time(const CIVMAP *pmap, int x, int y);
int civmap_transform_time(const CIVMAP *pmap, int x, int y);
int civmap_fortress_time(const CIVMAP *pmap, int x, int y);
int civmap_airbase_time(const CIVMAP *pmap, int x, int y);
int civmap_pollution_time(const CIVMAP *pmap, int x, int y);
int civmap_fallout_time(const CIVMAP *pmap, int x, int y);
#ifdef	FC__UNIT_H
int civmap_activity_time(const CIVMAP *pmap, int x, int y, 
  enum unit_activity activity);
#endif	FC__UNIT_H

enum special_type civmap_get_specials(const CIVMAP *pmap, int x, int y);
bool civmap_has_specials(const CIVMAP *pmap, int x, int y, 
  enum special_type spe);
void civmap_set_specials(const CIVMAP *pmap, int x, int y, 
  enum special_type spe);
void civmap_special_add(const CIVMAP *pmap, int x, int y, 
  enum special_type spe);
void civmap_special_delete(const CIVMAP *pmap, int x, int y, 
  enum special_type spe);
void civmap_infrastructure_delete(const CIVMAP *pmap, int x, int y);
void civmap_dirtiness_delete(const CIVMAP *pmap, int x, int y);

enum terrain_type civmap_get_terrain(const CIVMAP *pmap, int x, int y);
bool civmap_has_terrain(const CIVMAP *pmap, int x, int y, 
  enum terrain_type ter);
void civmap_set_terrain(const CIVMAP *pmap, int x, int y, 
  enum terrain_type ter);

void civmap_movecosts_init(const CIVMAP *pmap);
void civmap_movecosts_reset(const CIVMAP *pmap, int x, int y);
int civmap_movecost(const CIVMAP *pmap, struct unit *punit, int x, int y);

void civmap_irrigate(const CIVMAP *pmap, int x, int y);
void civmap_mine(const CIVMAP *pmap, int x, int y);
void civmap_transform(const CIVMAP *pmap, int x, int y);
void civmap_terrain_changed(const CIVMAP *pmap, int x, int y, 
  enum terrain_type prev);

int count_terrain_adjc(const CIVMAP *pmap, int x0, int y0, 
  enum terrain_type tt);
int count_special_adjc(const CIVMAP *pmap, int x0, int y0, 
  enum special_type spe);
bool is_terrain_adjc(const CIVMAP *pmap, int x0, int y0, enum terrain_type tt);
bool is_special_adjc(const CIVMAP *pmap, int x0, int y0, enum special_type spe);
bool is_special_close(const CIVMAP *pmap, int x0, int y0);
bool is_hut_close(const CIVMAP *pmap, int x0, int y0);
bool is_at_coast(const CIVMAP *pmap, int x0, int y0);
bool is_coast(const CIVMAP *pmap, int x0, int y0);
bool is_coastal(const CIVMAP *pmap, int x0, int y0);
bool is_water_adjc(const CIVMAP *pmap, int x0, int y0);
bool is_sea_usable(const CIVMAP *pmap, int x0, int y0);
bool is_terrain_clean(const CIVMAP *pmap, int x0, int y0);
bool is_starter_bad(const CIVMAP *pmap, int x0, int y0, int nr, int dist); 
bool can_reclaim_ocean(const CIVMAP *pmap, int x, int y);
bool can_channel_land(const CIVMAP *pmap, int x, int y);

void rand_civmap_pos(const CIVMAP *pmap, int *x, int *y);
void rand_region_pos(const CIVMAP *pmap, int *x, int *y, int x0, int y0, 
  int x1, int y1);
void rand_terrain_pos(const CIVMAP *pmap, int *x, int *y, int x0, int y0,
  int x1, int y1, int terrains, int specials, bool (*okuserfn)());
void rand_adjc(const CIVMAP *pmap, int *x, int *y, int x0, int y0);


/* These are used extensively, so are typically optimized to macros. */
#if !defined(FC__NO_CIVMAP_EX_MACROS) && !defined(FC__CIVMAP_EX_C)

#define civmap_get_tiles(pmap)      \
  ( (CIVTILE *)((pmap)->tiles) )
#define civmap_get_tile(pmap, x, y) \
  ( civmap_get_tiles(pmap) + civmap_pos_to_index((pmap), (x), (y)) )

#define	civmap_get_city(pmap, x, y)  \
  ( civmap_get_tile((pmap),(x),(y))->city )
#define	civmap_set_city(pmap, x, y, pcity)  \
  ( civmap_get_tile((pmap),(x),(y))->city = pcity )
#define	civmap_get_worked(pmap, x, y)  \
  ( civmap_get_tile((pmap),(x),(y))->worked )
#define	civmap_set_worked(pmap, x, y, pworked)  \
  ( civmap_get_tile((pmap),(x),(y))->worked = (pworked) )

#define	civmap_get_continent(pmap, x, y)  \
  ( civmap_get_tile((pmap),(x),(y))->continent )
#define	civmap_set_continent(pmap, x, y, val)  \
  ( civmap_get_tile((pmap),(x),(y))->continent = (val) )
#define	civmap_get_ocean(pmap, x, y)  \
  ( civmap_get_tile((pmap),(x),(y))->ocean )
#define	civmap_set_ocean(pmap, x, y, val)  \
  ( civmap_get_tile((pmap),(x),(y))->ocean = (val) )

#define	civmap_get_food(pmap, x, y)  \
  ( civtile_get_food(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_get_food_base(pmap, x, y)  \
  ( civtile_get_food_base(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_get_shield_base(pmap, x, y)  \
  ( civtile_get_shield_base(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_get_shield(pmap, x, y)  \
  ( civtile_get_shield(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_get_trade_base(pmap, x, y)  \
  ( civtile_get_trade_base(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_get_trade(pmap, x, y)  \
  ( civtile_get_trade(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_get_fpt_text(pmap, x, y)  \
  ( civtile_get_fpt_text(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_get_info_text(pmap, x, y)  \
  ( civtile_get_info_text(civmap_get_tile((pmap),(x),(y))) )

#define	civmap_road_time(pmap, x, y)  \
  ( civtile_road_time(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_railroad_time(pmap, x, y)  \
  ( civtile_railroad_time(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_mine_time(pmap, x, y)  \
  ( civtile_mine_time(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_irrigation_time(pmap, x, y)  \
  ( civtile_irrigation_time(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_transform_time(pmap, x, y)  \
  ( civtile_transform_time(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_fortress_time(pmap, x, y)  \
  ( civtile_fortress_time(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_airbase_time(pmap, x, y)  \
  ( civtile_airbase_time(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_pollution_time(pmap, x, y)  \
  ( civtile_pollution_time(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_fallout_time(pmap, x, y)  \
  ( civtile_fallout_time(civmap_get_tile((pmap),(x),(y))) )
#ifdef	FC__UNIT_H
#define	civmap_activity_time(pmap, x, y, activity)  \
  ( civtile_activity_time(civmap_get_tile((pmap),(x),(y)),(activity)) )
#endif	FC__UNIT_H

#define	civmap_get_specials(pmap, x, y)  \
  ( civtile_get_specials(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_has_specials(pmap, x, y, spe)  \
  ( civtile_has_specials(civmap_get_tile((pmap),(x),(y)), (spe)) )

#define	civmap_dirtiness_clear(pmap, x, y)  \
  ( civtile_dirtiness_clear(civmap_get_tile((pmap),(x),(y))) )

#define	civmap_get_terrain(pmap, x, y)  \
  ( civtile_get_terrain(civmap_get_tile((pmap),(x),(y))) )
#define	civmap_has_terrain(pmap, x, y, ter)  \
  ( civtile_has_terrain(civmap_get_tile((pmap),(x),(y)), (ter)) ) 

#endif	/* FC__NO_CIVMAP_EX_MACROS */

#endif	FC__CIVMAP_EX_H

