/**********************************************************************
 Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
 civtile_ex.c - Copyright (C) 2002 - R. Wetmore

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
***********************************************************************/

#define FC__CIVTILE_EX_C

/**
 * Extended functions for accessing and updating individual CIVTILEs,
 * in particular CIVTERRAIN accessor convenience methods.
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mem.h"
#include "support.h"

#include "civtile_ex.h"


// TODO: civtile_get_food/shield/trade including improvements and game
//       state like government ???

/***********************************************************************
  Return the tile food value. Convenience method for tiles
***********************************************************************/
int civtile_get_food(CIVTILE *ptile)
{
  return civterrain_type_get_food(civtile_get_terrain_type(ptile),
    ptile->special);
}

/***********************************************************************
  Return the basic food value. Convenience method for tiles
***********************************************************************/
int civtile_get_food_base(CIVTILE *ptile)
{
  return civterrain_type_get_food_base(civtile_get_terrain_type(ptile),
    ptile->special);
}

/***********************************************************************
  Return the tile shield value. Convenience method for tiles
***********************************************************************/
int civtile_get_shield(CIVTILE *ptile)
{
  return civterrain_type_get_shield(civtile_get_terrain_type(ptile),
    ptile->special);
}

/***********************************************************************
  Return the basic shield value. Convenience method for tiles
***********************************************************************/
int civtile_get_shield_base(CIVTILE *ptile)
{
  return civterrain_type_get_shield_base(civtile_get_terrain_type(ptile),
    ptile->special);
}

/***********************************************************************
  Return the tile trade value. Convenience method for tiles
***********************************************************************/
int civtile_get_trade(CIVTILE *ptile)
{
  return civterrain_type_get_trade(civtile_get_terrain_type(ptile),
    ptile->special);
}

/***********************************************************************
  Return the basic trade value. Convenience method for tiles
***********************************************************************/
int civtile_get_trade_base(CIVTILE *ptile)
{
  return civterrain_type_get_trade_base(civtile_get_terrain_type(ptile),
    ptile->special);
}


/***********************************************************************
  Return a (static) string with a tile's food/prod/trade
***********************************************************************/
const char *civtile_get_fpt_text(CIVTILE *ptile)
{
  static char s[64];
  const CIVTERRAIN_TYPE *pter = civtile_get_terrain_type(ptile);
  
  my_snprintf(s, sizeof(s), "%d/%d/%d",
	      civterrain_type_get_food(pter, ptile->special),
	      civterrain_type_get_shield(pter, ptile->special),
	      civterrain_type_get_trade(pter, ptile->special));
  return s;
}

/***********************************************************************
  Return a (static) string with terrain info; Convenience method for tiles
  eg: "Hills"
  eg: "Hills (Coals)"
  eg: "Hills (Coals) [Pollution]"
***********************************************************************/
const char *civtile_get_info_text(CIVTILE *ptile)
{
  return civterrain_type_get_info_text(civtile_get_terrain_type(ptile),
    ptile->special);
}


/***********************************************************************
  Convenience access method for tiles.
***********************************************************************/
int civtile_road_time(CIVTILE *ptile)
{
  return civterrain_type_road_time(civtile_get_terrain_type(ptile));
}

/***********************************************************************
  Convenience access method for tiles.
***********************************************************************/
int civtile_railroad_time(CIVTILE *ptile)
{
  return civterrain_type_railroad_time(civtile_get_terrain_type(ptile));
}

/***********************************************************************
  Convenience access method for tiles.
***********************************************************************/
int civtile_mine_time(CIVTILE *ptile)
{
  return civterrain_type_mine_time(civtile_get_terrain_type(ptile));
}

/***********************************************************************
  Convenience access method for tiles.
***********************************************************************/
int civtile_irrigation_time(CIVTILE *ptile)
{
  return civterrain_type_irrigation_time(civtile_get_terrain_type(ptile));
}

/***********************************************************************
  Convenience access method for tiles.
***********************************************************************/
int civtile_transform_time(CIVTILE *ptile)
{
  return civterrain_type_transform_time(civtile_get_terrain_type(ptile));
}

/***********************************************************************
  Convenience access method for tiles.
***********************************************************************/
int civtile_fortress_time(CIVTILE *ptile)
{
  return civterrain_type_fortress_time(civtile_get_terrain_type(ptile));
}

/***********************************************************************
  Convenience access method for tiles.
***********************************************************************/
int civtile_airbase_time(CIVTILE *ptile)
{
  return civterrain_type_airbase_time(civtile_get_terrain_type(ptile));
}

/***********************************************************************
  Convenience access method for tiles.
***********************************************************************/
int civtile_pollution_time(CIVTILE *ptile)
{
  return civterrain_type_pollution_time(civtile_get_terrain_type(ptile));
}

/***********************************************************************
  Convenience access method for tiles.
***********************************************************************/
int civtile_fallout_time(CIVTILE *ptile)
{
  return civterrain_type_fallout_time(civtile_get_terrain_type(ptile));
}

#ifdef	FC__UNIT_H
/***********************************************************************
  Time to complete the given activity on the given tile.
***********************************************************************/
int civtile_activity_time(CIVTILE *ptile, enum unit_activity activity)
{
  return civterrain_activity_time(civtile_get_terrain_type(ptile), activity);
}
#endif	/* FC__UNIT_H */


/***********************************************************************
  Carry out the activity and return the previous terrain_type or T_LAST
  if it has not changed materially.
***********************************************************************/
int civtile_road(CIVTILE *ptile)
{
  enum terrain_type now = ptile->terrain;

  /* Simple road/rail */
  if ((ptile->special & S_ROAD)) {
    civtile_special_add(ptile, S_RAILROAD);
  } else {
    civtile_special_add(ptile, S_ROAD);
  }
  /* reset is required */
  return now;
}

/***********************************************************************
  Carry out the activity and return the previous terrain_type or T_LAST
  if it has not changed.
***********************************************************************/
int civtile_irrigate(CIVTILE *ptile)
{
  enum terrain_type now, result;
  const CIVTERRAIN_TYPE *terraintypes = ptile->global->terrain->types;
  
  now = ptile->terrain;
  result = terraintypes[now].irrigation_result;

  if (now == result) {
    /* Simple irrigation */
    if ((ptile->special & S_IRRIGATION)) {
      civtile_special_add(ptile, S_FARMLAND);
    } else {
      if ((ptile->special & S_MINE)) {
        civtile_special_delete(ptile, S_MINE);
      }
      civtile_special_add(ptile, S_IRRIGATION);
    }
    /* no reset is required */
    return T_LAST;
  }

  if (result < T_LAST) {
    /* transform */
    ptile->terrain = result;
    if (now != T_OCEAN) {
      /* destroy selective current specials if result doesn't support */
      if (terraintypes[result].mining_result != result)
        civtile_special_delete(ptile, S_MINE);
      if (terraintypes[result].irrigation_result != result)
        civtile_special_delete(ptile, S_FARMLAND | S_IRRIGATION);
    }
    if (result == T_OCEAN) {
      civtile_infrastructure_delete(ptile);
      civtile_dirtiness_delete(ptile);
      /* FIXME: When code can handle rivers in oceans, don't clear this! */
      civtile_special_delete(ptile, S_RIVER);
    }
    /* reset is required (e.g. for move_costs) */
    return now;
  }

  return T_LAST;
}

/***********************************************************************
  Carry out the activity and return the previous terrain_type or T_LAST
  if it has not changed.
***********************************************************************/
int civtile_mine(CIVTILE *ptile)
{
  enum terrain_type now, result;
  const CIVTERRAIN_TYPE *terraintypes = ptile->global->terrain->types;
  
  now = ptile->terrain;
  result = terraintypes[now].mining_result;

  if (now == result) {
    /* Simple mining */
    civtile_special_delete(ptile, S_FARMLAND | S_IRRIGATION);
    civtile_special_add(ptile, S_MINE);
    /* no reset is required */
    return T_LAST;
  }

  if (result < T_LAST) {
    /* transform */
    ptile->terrain = result;
    /* TODO: only destroy selective current specials by from and to type??? */
    if (now != T_OCEAN) {
      /* destroy selective current specials if result doesn't support */
      if (terraintypes[result].mining_result != result)
        civtile_special_delete(ptile, S_MINE);
      if (terraintypes[result].irrigation_result != result)
        civtile_special_delete(ptile, S_FARMLAND | S_IRRIGATION);
    }
    if (result == T_OCEAN) {
      civtile_infrastructure_delete(ptile);
      civtile_dirtiness_delete(ptile);
      /* FIXME: When code can handle rivers in oceans, don't clear this! */
      civtile_special_delete(ptile, S_RIVER);
    }
    /* reset is required (e.g. for move_costs) */
    return now;
  }

  return T_LAST;
}

/***********************************************************************
  Carry out the activity and return the previous terrain_type or T_LAST
  if it has not changed.
  FIXME:  see server/settlers.c: ai_calc_transform(), unittools.c()
          for all the land/ocean side effects.
***********************************************************************/
int civtile_transform(CIVTILE *ptile)
{
  enum terrain_type now, result;
  const CIVTERRAIN_TYPE *terraintypes = ptile->global->terrain->types;
  
  now = ptile->terrain;
  result = terraintypes[now].transform_result;

  if (now == result) {
    /* no reset is required - actually this is probably an error */
    return T_LAST;
  }

  if (result < T_LAST) {
    /* transform */
    ptile->terrain = result;
    /* TODO: only destroy selective current specials by from and to type??? */
    if (now != T_OCEAN) {
      /* destroy selective current specials if result doesn't support */
      if (terraintypes[result].mining_result != result)
        civtile_special_delete(ptile, S_MINE);
      if (terraintypes[result].irrigation_result != result)
        civtile_special_delete(ptile, S_FARMLAND | S_IRRIGATION);
    }
    if (result == T_OCEAN) {
      civtile_infrastructure_delete(ptile);
      civtile_dirtiness_delete(ptile);
      /* FIXME: When code can handle rivers in oceans, don't clear this! */
      civtile_special_delete(ptile, S_RIVER);
    }
    /* reset is required (e.g. for move_costs) */
    return now;
  }

  return T_LAST;
}

